import java.util.concurrent.locks.*;

class BoundedBuffer {
    final Lock lock = new ReentrantLock(); // We use the lock to guarantee mutual exclusion in the methods of the monitor
                                           // Wrap all methods with -- lock.lock() try {} finally{lock.unlock()}
    final Condition notFull = lock.newCondition(); // Creates a condition variable for threads to wait
    final Condition notEmpty = lock.newCondition(); // Creates a condition variable for threads to wait
    final int BUFFER_SIZE = 5;

    final Integer[] buffer = new Integer[BUFFER_SIZE];
    int current = 0;

    public void produce(Integer x) throws InterruptedException {
	lock.lock();
	try {
	    while(current == buffer.length){
		notFull.await();
	    }
	    buffer[current] = x;
	    System.out.println("I've produced the value " + x);
	    current++;
	    if (current >= 1) {
		notEmpty.signal();
	    }
	} finally {
	    lock.unlock();
	}
    }

    public Integer consume() throws InterruptedException { 
	Integer x = 0;
	lock.lock();
	try {
	    while(current == 0){
		notEmpty.await();
	    }
	    current--;
	    x = buffer[current];
	    System.out.println("I've consumed the value " + x);
	    if (current <= buffer.length - 1) {
		notFull.signal();
	    }
	    return x;
	} finally {
	    lock.unlock();
	}
    }
    
}
